""" This script file is the main script file which is the front interface of this app
and also all other functionality of other modules can be accessed by the GUI interface
it creates. This module basically creates the directory where the App content will
be stored and also responsible for the support to every other modules.
"""

import ctypes
import os
import sys
import time
import tkinter as tk
from sys import platform

import appdirs
import requests
from windows_wifi_manager import top_level_window as db
from windows_wifi_manager.display_data import WifiDisplayBox
from windows_wifi_manager.wifi_connection import SystemWifiConnection

SOFTWARE_NAME = "SavedPasswordFinder"
SOFTWARE_AUTHOR = "NihalAgarwal"
APP_DIR = appdirs.user_data_dir(SOFTWARE_NAME, SOFTWARE_AUTHOR)

if not os.path.isdir(APP_DIR):
    os.makedirs(APP_DIR)


def main():
    """ Main Method"""

    # THE BELOW FUNCTIONS ARE THE FUNCTION WHICH ARE CALLED BY THE WIDGETS
    # OR TRIGGERED BY THE WIDGETS

    def about():
        """ Describe about the software and what it does and also contact
        details to developer, when user or end user face any issue.
        """
        db.About(main_window)

    def update():
        """ Check if software has latest version or not"""
        try:
            time.sleep(4)
            _ = requests.get('http://www.google.com/', timeout=5)
            db.MessageBox(main_window, "Version: 0.2.0 v \n\n No Update Available!", "warning")
        except requests.ConnectionError:
            db.MessageBox(main_window, "Check Your Internet Connection", "error")

    def add_profile():
        """ Add new Wi-Fi profile to the system."""

        db.AddProfile(main_window, APP_DIR)

    def refresh_treeview():
        """ Refresh the software and search for all saved networks and
        add it to the TreeView.
        """
        temp = wdb.refresh_treeview()  # Generate a new list of saved Wi-Fi

        if temp:
            db.MessageBox(main_window, 'Refresh Successfully.', 'check')

        else:
            db.MessageBox(main_window, 'Facing issue.')

    def delete_profile():
        """ Call delete_profile method of WifiDisplayBox to delete a wifi profile."""
        wdb.delete_profile(main_window)

    def on_exiting():
        """ End the process and quit the software"""

        result = db.QuitWindow(main_window)
        if result.temp.get():
            for i in os.listdir(APP_DIR):
                if i != 'temp_':
                    os.remove(APP_DIR + "\\" + i)
            main_window.destroy()

    def find_network():
        """ Check if system is connect to any network. If yes the
        network name is return and button value is modified to 'refresh'
        (if no network connected) otherwise button value is modified to
        'disconnect' and label is updated as network name.
        """
        ssid_name = system_wifi_connection.is_connected()

        if ssid_name is not None:
            refresh_disconnect_button.configure(text="Disconnect",
                                                command=disconnect_button)
            current_network_label.configure(text=ssid_name, foreground="green")

        else:
            refresh_disconnect_button.configure(text="Refresh",
                                                command=find_network)
            current_network_label.configure(text="No Network", foreground="red")

    def disconnect_button():
        """ Disconnect form the current network. """
        disconnected = system_wifi_connection.disconnect_connection()
        if disconnected:
            refresh_disconnect_button.configure(text="Refresh",
                                                command=find_network)
            current_network_label.configure(text="No Network", foreground="red")
            text = "You had to manually connect to same or different network."
            db.MessageBox(main_window, text, "warning")  # pop-up window

    # Checking OS
    if platform != "win32":
        print("The package or Application is made only for window users. ")
        print("The package development for other OS is still in development. ")
        sys.exit(0)

    # Creating and configuring Main Window
    main_window = tk.Tk()
    main_window.configure(background='white', highlightbackground="grey")
    main_window.title("Windows Wi-Fi Manager")
    try:
        # the below line of code will avoid blurry looking GUI by increasing DPI.
        ctypes.windll.shcore.SetProcessDpiAwareness(1)
        main_window.geometry("900x600")

    except Exception:
        main_window.geometry("900x500")

    main_window.minsize(800, 350)
    path_dir = os.path.dirname(os.path.realpath(__file__))
    main_window.iconbitmap(path_dir + "/data/images/wifi2.ico")

    # Creating MENU and SUBMENU
    menu = tk.Menu(main_window)
    submenu = tk.Menu(main_window, tearoff=0)
    main_window.config(menu=menu)
    menu.add_cascade(label="Help", menu=submenu)
    submenu.add_command(label="About", command=about)
    submenu.add_separator()
    submenu.add_command(label="Check for updates", command=update)
    submenu.add_separator()
    submenu.add_command(label="Quit", command=on_exiting)

    # Theme for Buttons
    style = tk.ttk.Style()
    style.configure('TButton', font=("Playfair Display", 15))
    style.map('TButton', font=[('active', ("Playfair Display", 12))],
              background=[('active', 'blue')])

    # Frame1 for system connected network display
    top_horizontal_frame = tk.Frame(main_window)
    # Frame2 for TreeView
    tree_view_frame = tk.Frame(main_window, background='white')
    # Frame3 for refresh, delete, add profile.
    vertical_button_frame = tk.Frame(main_window, background='white')

    # Create object to easily reference its method and fields(e.g., SSID name)
    system_wifi_connection = SystemWifiConnection(main_window)

    # Displays the heading(title)
    heading_label = tk.Label(top_horizontal_frame, text="Current Network: ",
                             font=("Playfair Display", 13, "bold"))

    # Button for disconnection or refresh
    refresh_disconnect_button = tk.ttk.Button(top_horizontal_frame, style='TButton')

    # Displays network name
    current_network_label = tk.Label(top_horizontal_frame,
                                     font=("Playfair Display", 13, "bold"))

    # packing
    heading_label.pack(padx=10, pady=10, side=tk.LEFT)
    current_network_label.pack(padx=10, pady=10, side=tk.LEFT)
    refresh_disconnect_button.pack(padx=10, pady=10, side=tk.LEFT)
    find_network()

    # Defining and packing buttons for frame2
    refresh_button = tk.ttk.Button(vertical_button_frame,
                                   text="Refresh List", style='TButton',
                                   cursor="hand2", command=refresh_treeview)

    delete_button = tk.ttk.Button(vertical_button_frame,
                                  text="Delete WiFi", style='TButton',
                                  cursor="hand2", command=delete_profile)

    add_profile_button = tk.ttk.Button(vertical_button_frame,
                                       text="Add Profile", style='TButton',
                                       cursor="hand2", command=add_profile)

    # Creating TreeView and packing it to the frame2
    wdb = WifiDisplayBox(APP_DIR, tree_view_frame)

    # Packing all three buttons( refresh, delete, add_profile)
    refresh_button.pack(side=tk.TOP, pady=25, padx=10, anchor="center")
    delete_button.pack(side=tk.TOP, pady=25, padx=10, anchor="center")
    add_profile_button.pack(side=tk.TOP, pady=25, padx=10, anchor="center")

    # Packing frame1, frame2 and frame3

    top_horizontal_frame.grid(row=0, column=0, columnspan=2, sticky='nsew')
    vertical_button_frame.grid(row=1, column=1, sticky='nsew')
    tree_view_frame.grid(row=1, column=0, sticky='nsew')
    main_window.grid_columnconfigure(0, weight=1)
    main_window.grid_rowconfigure(1, weight=1)

    main_window.protocol("WM_DELETE_WINDOW", on_exiting)
    main_window.mainloop()


if __name__ == '__main__':
    main()



""" This script file is responsible to return the check whether system is
connected to any Wi-Fi network and return SSID name and also give functionality
to disconnect from the current connected network.
"""
import re
import subprocess

from windows_wifi_manager.top_level_window import MessageBox


class SystemWifiConnection:
    """ Check system is connected to any Wi-Fi network and also the functionality
    of disconnect and refresh(to check whether system is connected to any network or
    not.
    """

    def __init__(self, parent):
        """ Declare ssid_name, interface_name(Wi-Fi), parent(reference to parent window
        which is used to display error pop-up message.
        """

        self.parent = parent
        self.interface_name = None
        self.ssid_name = None

    def is_connected(self):
        """ Check if system is connected to any Wi-Fi network. If true,
        return ssid_name and interface name else return None.
        """

        command = 'netsh wlan show interfaces | findstr "Name State SSID"'
        output = subprocess.run(command, shell=True, capture_output=True, text=True)
        my_regex = r"Name.*?connected.*?(?= *BSSID)"
        network_status = re.search(my_regex, output.stdout, re.DOTALL)

        if network_status is None:
            return None
        network_status = network_status.group()
        self.interface_name = re.findall(r"[\n\r-<>():]?.*Name\s*: ([^\n\r]*)", network_status)[0]
        self.ssid_name = re.findall(r"[\n\r-<>():]?.*SSID\s*: ([^\n\r]*)", network_status)[0]
        return self.ssid_name

    def disconnect_connection(self):
        """ Disconnects if system is connected to any Wi-Fi network"""

        command = 'netsh wlan disconnect interface = "' + self.interface_name + '"'
        output = subprocess.run(command, shell=True, capture_output=True, text=True)

        # Check if system is disconnected from network successfully.
        if output.returncode != 0:
            message = "Sorry, unable to disconnect"
            MessageBox(self.parent, message, "error")  # Display message of error.
            return False
        return True

    # WARNING
    # There is a bug in below function i.e., when user disconnect and reconnect to same network
    # and network is not available anymore. It still shows the result that connection to same
    # network is done successfully. It is a bug of 'command prompt'.
    def reconnect_network(self):
        """ Try to reconnect to same network. """
        command = 'netsh wlan connect name="' + self.ssid_name + '" interface = "' \
                  + self.interface_name + '"'
        output = subprocess.run(command, shell=True, capture_output=True, text=True)

        # Check if system reconnect to same network successfully.
        if output.returncode != 0:
            message = "Unable to reconnect(either previous network is not available" \
                      " in range or profile is deleted)"
            MessageBox(self.parent, message, "error")  # Display message of error.
            return False
        return True



"""This module takes the name of the profile and return the Wi-Fi details like SSID name,
Authentication and Key. Generating the Saved Wi-Fi details of some special SSID's gives a
problem. For a different type of SSID name, like some SSID name contain 'emojis' while some
contain special symbols which create a lot of problems when dealing with command prompt. So,
generating XML file of the per profile doesn't resolve some problem but not all. Some SSID's
like 'Redm"i=" are still creating problems.
"""

import os
import re
import subprocess
import xml.etree.ElementTree as etree


class WifiInformation:
    """ This class generates and return the SSID, Authentication, Key
    for the Wi-Fi profile as given as input.
    """

    def __init__(self, app_path):

        """
            This method is used to check if the necessary folder, file is present,
        otherwise create it. Also, initialize object variables.
        """

        self.app_path = app_path

        if not os.path.isdir(self.app_path + "\\temp_"):
            os.mkdir(self.app_path + "\\temp_")

        if os.path.isfile(self.app_path + "/Saved Wifi list.txt"):
            os.remove(self.app_path + "/Saved Wifi list.txt")
            self.generating_wifi_list()

        else:
            self.generating_wifi_list()

    def generating_wifi_list(self):
        """ Generates the list of Wi-Fi saved in you system and save it in parent folder of temp_.
        """

        with open(self.app_path + "/Saved Wifi list.txt", "w") as file:

            output = subprocess.run("netsh wlan show profile", shell=True,
                                    capture_output=True, text=True)

            # Parsing and finding required information and saving it into the file.
            if output.returncode == 0:
                my_regex = r"[\n\r-<>():].*All User Profile\s*: ([^\n\r]*)"
                list_of_wifi = re.findall(my_regex, output.stdout)
                list_of_wifi.sort(key = lambda x: x.lower())
                for i in list_of_wifi:
                    file.write(i + "\n")

    def retrieving_list_of_wifi(self):
        """ The below code returns list of tuple in the form of [(name, "", "", ""),].
        """

        list_of_wifi_detail = []
        with open(self.app_path + "/Saved Wifi list.txt", 'r') as file:
            wifi_detail = file.readlines()
            for item in wifi_detail:
                list_of_wifi_detail.append((item[:-1], "", "", ""))
        return list_of_wifi_detail

    def wifi_details_(self, name, text_file_name, wifi_detail):
        """ Called if XML or text file for particular wifi doesn't exist.
         It tries another possible way to retrieve information.
         """

        # Getting information for specific profile using
        # netsh wlan show profile name= "name" key =clear
        # then using findstr to search
        cmd = 'netsh wlan show profile name="' + name + \
              '" key=clear | findstr "Name name Authentication Key"'
        output = subprocess.run(cmd, shell=True, capture_output=True, text=True)

        # If command executed successfully.
        if output.returncode == 0:
            ssid_regex = r"[\n\r-<>():].*Name\s*: ([^\n\r]*)"
            ssid_name = re.findall(ssid_regex, output.stdout)[0]
            authentication_regex = r"[\n\r-<>():].*Authentication\s*: ([^\n\r]*)"
            authentication = re.findall(authentication_regex, output.stdout)[0]

            if authentication == "Open":
                key_content = "None"
            else:
                key_content_regex = r"[\n\r-<>():].*Key Content\s*: ([^\n\r]*)"
                key_content = re.findall(key_content_regex, output.stdout)[0]

            return self.data_saving(text_file_name, ssid_name, authentication,
                                    key_content, wifi_detail)

        # THIS LAST OPTION DOESN'T PROVIDE THE KEY FOR WIFI. IT ONLY PROVIDE SSID's
        # AND AUTHENTICATION BECAUSE 'netsh wlan show profile * key=clear' PROVIDES
        # EVERYTHING FOR SPECIFIC PROFILE EXCEPT KEY. THE BELOW CODE IS USED TO RETRIEVE
        # INFORMATION FOR SSID's LIKE "Red"mi=", "abc:>gh=", etc. IF ANY ONE CAN SOLVE
        # THE ISSUE THEN YOU CAN BY REMOVING THE CODE FROM ELSE BLOCK AND PUT YOUR OWN
        # CODE, BUT DON'T REMOVE THE RETURN STATEMENT.

        try:
            cmd = 'netsh wlan show profile * key =clear | findstr ' \
                  '"Name name Authentication Key"'
            output = subprocess.run(cmd, shell=True, capture_output=True, text=True)
            if output.returncode != 0:
                raise Exception
            my_regex = name + r".*?(?=Name)"
            info_of_ssid = re.findall(my_regex, output.stdout, re.DOTALL)[0]
            ssid_name_regex = r"[\n\r-<>():].*SSID name\s*: ([^\n\r]*)"
            ssid_name = re.findall(ssid_name_regex, info_of_ssid)[0]
            authentication_regex = r"[\n\r-<>():].*Authentication\s*: ([^\n\r]*)"
            authentication = re.findall(authentication_regex, info_of_ssid)[0]
            key_content = "*Unable to find*"

        except Exception:
            ssid_name = name
            authentication = "*Unable to find*"
            key_content = "*Unable to find*"

        return self.data_saving(text_file_name, ssid_name, authentication,
                                key_content, wifi_detail)

    @staticmethod
    def xml_parsing(tree, wifi_detail):
        """Parsing XML file to retrieve ssid, authentication and key(if any).
        """

        root = tree.getroot()
        namespace = re.match(
            r"{[^\s\n\r]+}", root.tag).group()

        ssid = tree.find(".//" + namespace + "SSID")
        ssid = ssid.find(".//" + namespace + "name")
        wifi_detail.append(ssid.text)
        authentication = tree.find(".//" + namespace + "authentication")
        wifi_detail.append(authentication.text)
        if authentication.text != "open":
            key_content = tree.find(".//" + namespace + "keyMaterial")
            wifi_detail.append(key_content.text)
        else:
            wifi_detail.append(None)

        return wifi_detail

    def wifi_details(self, name):
        """ Using the 'name' parameter find the profile and check every possible way
        to retrieve SSID, authentication and key(if any).
        """

        wifi_detail = [name]
        text_file_name = str(sum([ord(i) for i in name]))

        # if xml file of particular wifi profile is there in temp folder
        if os.path.isfile(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml"):
            tree = etree.parse(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml")

            return self.xml_parsing(tree, wifi_detail)

        # if text file of particular wifi profile is there in temp folder
        elif os.path.isfile(self.app_path + "\\temp_\\" + text_file_name + ".txt"):
            return self.data_retrieval(text_file_name, wifi_detail)

        else:
            # export all profile data into their specified xml file and save
            # to specific folder which is path to temp_ folder

            cmd = 'netsh wlan export profile key=clear folder="' + self.app_path + '\\temp_"'
            output = subprocess.run(cmd, shell=True, capture_output=True, text=True)

            if output.returncode != 0:
                return self.wifi_details_(name, text_file_name, wifi_detail)

            # If xml file of particular wifi profile is there in temp folder
            elif os.path.isfile(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml"):
                tree = etree.parse(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml")
                return self.xml_parsing(tree, wifi_detail)

            return self.wifi_details_(name, text_file_name, wifi_detail)

    def data_retrieval(self, text_file_name, wifi_detail):
        """ Getting SSID name, authentication and password from the text file."""

        with open(self.app_path + "\\temp_\\" + text_file_name + ".txt", "r") as file:
            wifi_detail.append(file.readline()[:-1])
            auth_temp = file.readline()[:-1]
            wifi_detail.append(auth_temp)
            if auth_temp != "*None*":
                wifi_detail.append(file.readline()[:-1])
            else:
                wifi_detail.append(None)
            return wifi_detail

    def data_saving(self, text_file_name, ssid, authentication, key, wifi_detail):
        """ Saving SSID name, authentication and password to text file."""

        with open(self.app_path + "\\temp_\\" + str(text_file_name) + ".txt", "w") as file:
            file.writelines(ssid.strip('""') + "\n")
            file.writelines(authentication + "\n")
            wifi_detail.append(ssid.strip('""'))
            wifi_detail.append(authentication)
            if authentication != "Open" or authentication != "open":
                file.writelines(key + "\n")
                wifi_detail.append(key)
            else:
                file.writelines("*None*" + "\n")
                wifi_detail.append(None)

        return wifi_detail


# used for testing
if __name__ == "__main__":
    PATH = "C:\\Users\\Nihal\\AppData\\Local\\NihalAgarwal\\SavedPasswordFinder"
    OBJ1 = WifiInformation(PATH)
    # Display the list of tuple in the form of [('name', "", "", ""),]
    print(OBJ1.retrieving_list_of_wifi())
    # Display the list of detail for the given wifi.
    print(OBJ1.wifi_details("mr. n="))



""" This module is responsible to create Tree View where save wifi in the
system of user will displayed with other data like (SSID name, Authentication,
Key) and also having functionality to delete the Wi-Fi profile from the system
and refresh the content of the TreeView."""

import os
import subprocess
import tkinter as tk
import tkinter.font as tkfont
import tkinter.ttk as ttk
import time

from . import top_level_window as db
from . import wifi_data as saved_wifi_info


class WifiDisplayBox:
    """ Creates a TreeView and display all the saved wifi and their
    SSID's, Authentication and Password.
    """

    def __init__(self, app_path, frame):
        self.frame = frame
        self.tree_view = None
        self.list_of_children = None
        self.app_path = app_path
        self.getting_data_obj = GettingData(app_path)
        self.create_treeview()
        self.build_tree()
        self.index_of_previous_item = ("", "", False)

    def get_wifi_list(self):
        """ Called generating_wifi_list() function of SavedInfoFile."""
        return self.getting_data_obj.wifi_list()

    def create_treeview(self):
        """ Create tree view and configure style for it."""

        # Defining styling for TreeView.
        style = ttk.Style(self.frame)
        style.configure("Treeview.Heading", font=("Playfair Display", 11,
                                                  'bold'), foreground='#6C7A89')
        style.configure("Treeview", font=('Calibri', 11))
        style.map("Treeview.Heading",
                  font=[('pressed', ("Playfair Display", 9, 'bold'))],
                  foreground=[('pressed', '#264348'), ('active', '#264348')])

        self.tree_view = ttk.Treeview(self.frame, columns=self.getting_data_obj.get_headings(),
                                      show="headings")
        headings_treeview = self.getting_data_obj.get_headings()

        # Defining Headings to the TreeView and defining column width.
        for i in enumerate(headings_treeview):
            self.tree_view.heading(i[0], text=i[1], anchor=tk.NW)
            self.tree_view.column(i[0], width=tkfont.Font().measure(i[1].title()),
                                  minwidth=130, anchor=tk.NW)

        # Adding and Scrollbars to the TreeView.
        vertical_scrollbar = ttk.Scrollbar(orient="vertical", command=self.tree_view.yview)
        horizontal_scrollbar = ttk.Scrollbar(orient="horizontal", command=self.tree_view.xview)
        self.tree_view.configure(yscrollcommand=vertical_scrollbar.set)
        self.tree_view.configure(xscrollcommand=horizontal_scrollbar.set)
        self.tree_view.grid(column=0, row=0, sticky='nsew', in_=self.frame)
        vertical_scrollbar.grid(column=1, row=0, sticky='ns', in_=self.frame)
        horizontal_scrollbar.grid(column=0, row=1, sticky='ew', in_=self.frame)

        self.frame.grid_columnconfigure(0, weight=1)
        self.frame.grid_rowconfigure(0, weight=1)

    def build_tree(self):
        """ Adding the list of wifi to the treeview in the form of ["name", "", "" ,""]."""

        # Add rows to the TreeView with the column[0] = name
        for item in self.get_wifi_list():
            self.tree_view.insert('', 'end', values=item, tags='ttk')

        # binding the functions
        self.tree_view.tag_bind('ttk', sequence="<<TreeviewSelect>>", callback=self.treeview_select)
        self.list_of_children = list(self.tree_view.get_children())

    def treeview_select(self, _=None):
        """ Function is called when user select any of the row in TreeView and displays
        the SSID, Authentication, Key of the selected row and deletes the SSID,
        Authentication and Key of previous selected row.
        """

        # get id of the selected row.
        item_id = self.tree_view.focus()

        # If user move from previous selected row to newly selected row, it removes
        # the SSId, Authentication, Key of previously selected row(except name).
        if self.index_of_previous_item[-1]:
            values_ = [self.index_of_previous_item[1], "", "", ""]
            self.tree_view.item(self.index_of_previous_item[0], values=values_)

        # The below code displays the SSID, Authentication, Key of newly selected row
        name = self.tree_view.item(item_id)['values'][0]
        current_item_detail = self.getting_data_obj.detailed_list(name)
        self.tree_view.item(item_id, values=[*current_item_detail])
        self.index_of_previous_item = (item_id, name, True)

    def refresh_treeview(self):
        """ Refresh the treeView and add or remove the profile from the TreeView
        if it is not present any more or newly added respectively.
        """

        # If in a TreeView any row is selected then deselect it.
        try:
            if self.tree_view.focus():
                self.tree_view.item(self.index_of_previous_item[0],
                                    values=[self.index_of_previous_item[-2], "", "", ""])
                self.index_of_previous_item = ("", "", False)

            if os.path.isfile(self.app_path + "\\Saved Wifi list.txt"):
                os.remove(self.app_path + "\\Saved Wifi list.txt")

            # clearing the whole treeview
            for i in self.list_of_children:
                self.tree_view.delete(i)

            # Generating fresh newly wifi list.
            self.getting_data_obj.create_wifi_list()
            # Modify rows of tree view and remove or add newly added data.
            self.build_tree()
            time.sleep(2)
            return True

        except Exception:
            time.sleep(2)
            return

    def delete_profile(self, parent_window):
        """ Delete Wi-Fi profile and all its content from the System."""

        try:
            if len(self.list_of_children) < 1:
                text = "No item available to delete."
                db.MessageBox(parent_window, text, "error")
            # Return id and name of selected or highlighted row
            iid = self.tree_view.focus()
            name = self.tree_view.item(iid)['values'][0]
            index_ = self.list_of_children.index(iid)  # return the index of the iid in the list

            # Delete the wifi profile from tree view as well as from system
            self.tree_view.delete(iid)
            self.list_of_children.pop(index_)
            self.index_of_previous_item = ("", "", False)

            # Removes the Wi-Fi profile from the user system.
            command = 'netsh wlan delete profile name="' + name + '"'
            subprocess.run(command, shell=True, capture_output=True, text=True)

            # Remove the xml or text file also, which is saved in temp_ directory.
            text_file_name = str(sum([ord(i) for i in name]))
            if os.path.isfile(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml"):
                os.remove(self.app_path + "\\temp_\\Wi-Fi-" + name + ".xml")

            elif os.path.isfile(self.app_path + "\\temp_\\" + text_file_name + ".txt"):
                os.remove(self.app_path + "\\temp_\\" + text_file_name + ".txt")

            # When the selected row deleted, the below code decides where to move the focus next
            # means to select which row.
            if len(self.list_of_children) >= 1:
                if index_ == len(self.list_of_children):
                    index_ = index_ - 1
                self.tree_view.selection_set(self.list_of_children[index_])
                self.tree_view.focus(self.list_of_children[index_])

            # Pop-Up message to display that profile successfully deleted.
            text = "Successfully deleted."
            db.MessageBox(parent_window, text, "warning")

        except Exception:
            self.index_of_previous_item = ("", "", False)
            text = "Unable to delete. Software is facing several issue during deletion."
            db.MessageBox(parent_window, text, "error")


class GettingData:
    """ This is a helping class which will help WifiDisplayBox to call methods of SavedWifiInfo and
    reformat and parse some of the results return by methods of SavedWifiInfo.
    """

    def __init__(self, app_path):
        self.app_path = app_path
        self.wifi_information = saved_wifi_info.WifiInformation(self.app_path)
        self.headings_of_treeview = ['Wifi Name', 'SSID', 'Authentication', 'Password']

    def create_wifi_list(self):
        """ Return newly created wifi list."""
        self.wifi_information.generating_wifi_list()

    def get_headings(self):
        """ Return list of heading names """
        return self.headings_of_treeview

    def wifi_list(self):
        """ Return wifi data of specific profile in the form of
        ["name", "" ,"", ""]
        """
        return self.wifi_information.retrieving_list_of_wifi()

    def detailed_list(self, name):
        """ Return wifi data of specific profile in the form of
        ["name", "ssid" ,"authentication", "key"]
        """
        profile_detail = self.wifi_information.wifi_details(name)
        if profile_detail[-1] is None or profile_detail[-1] == "*None*":
            profile_detail[-1] = ""
        return profile_detail



from setuptools import setup

from windows_wifi_manager import __version__ as version

setup(
    name='windows-wifi-manager',
    version=version,
    packages=['windows_wifi_manager'],
    url='https://github.com/NihalAgarwal/Windows-Wi-Fi-Manager',
    license='MIT',
    author='Nihal Agarwal',
    author_email='nihal.agarwal.1426@gmail.com',
    description='Know the saved WiFi details and add WiFi profile using a'
                ' simple graphical interface in windows.',
    long_description=''.join(open('README.md', encoding='utf-8').readlines()),
    long_description_content_type='text/markdown',
    keywords=['gui', 'wifi', 'netsh', 'Wi-Fi'],
    include_package_data=True,
    install_requires=['appdirs==1.4.3', 'lxml==4.4.2', 'requests==2.22.0'],
    python_requires='>=3.7',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: Microsoft :: Windows',
    ],
    entry_points={
        'console_scripts': [
            'windowswifimanager=windows_wifi_manager.__main__:main',
            'windows-wifi-manager=windows_wifi_manager.__main__:main'
        ],
    },
)



"""This module creates different top-level window, creates and placing widget and adding
functionality to the widgets. Every class defined here creates a different top level window
with different functionality and uses. Like one class 'QuitWindow' creates a top level window
and asking user if they really want to quit(like a message box). Some top-level widgets work
can easily be done by tkinter messagebox but due to position or geometry cannot be specified
explicitly and the message box (top-level window) created by some of this classes places the
window at the centre of the parent window (irrespective of what the size of the parent window
is."""
import os
import tkinter
import webbrowser
from abc import ABC, abstractmethod
from tkinter import ttk
from xml.dom import minidom

try:
    import lxml.etree as etree
except ImportError:
    import xml.etree.ElementTree as etree
import subprocess


class BasicDialog(tkinter.Toplevel, ABC):
    """ This is a base class of every top level window, it only defines the geometry
        of the top level window, move focus to top level window, provide basic design
        to the window and prevent user to access parent window until this top level window
        closes."""

    def __init__(self, parent, title=None):
        tkinter.Toplevel.__init__(self, parent)
        self.transient(parent)

        # Defines the position w.r.t to parent window, so that
        # top-level window will be in the middle of parent window
        #
        # The below x, y takes time for calculation(i.e., the reason for
        # flashes in top level window, but it is necessary because it guide
        # where to position the top level window as a reference to parent.
        # or some other functions is doing issue
        #
        x_coordinate = parent.winfo_rootx() + parent.winfo_width() // 2 - \
                       self.winfo_reqwidth() // 2  # x-coordinate
        y_coordinate = parent.winfo_rooty() + parent.winfo_height() // 2 - \
                       self.winfo_reqheight() // 2  # y-coordinate
        self.geometry("+%d+%d" % (x_coordinate, y_coordinate))
        self.configure(background='white')
        if title:
            self.title(title)
        self.iconbitmap(os.path.dirname(os.path.realpath(__file__)) + "/data/images/wifi2.ico")
        self.resizable(False, False)  # window cannot be resized
        self.temp = tkinter.BooleanVar()  # Defining Variable of type boolean
        self.parent = parent
        body_frame = tkinter.Frame(self, background='white')
        self.initial_focus = self.body(body_frame)  # setting focus to the frame ssid_entry
        body_frame.pack(padx=5, pady=5)
        self.button_box()
        self.grab_set()

        # set focus to either and widget or to the window itself
        if not self.initial_focus:
            self.initial_focus = self
        self.bind("<Button-1>", self.alarm)
        self.bind("<Button-3>", self.alarm)
        self.protocol("WM_DELETE_WINDOW", self.cancel)
        self.initial_focus.focus_set()
        self.wait_window(self)  # wait until top level closes

    def alarm(self, event):
        """ This function will capture the mouse click coordinate (both left or click)
            as respect to system screen and if the mouse click lies in parent window
            (not in top level window), it flashes out the top level window and the
             window bell will ring."""

        # top left corner of top level window
        x1_coordinate, y1_coordinate = self.winfo_rootx(), self.winfo_rooty()

        # bottom right corner of top level window
        x2_coordinate = x1_coordinate + self.winfo_width()
        y2_coordinate = y1_coordinate + self.winfo_height()
        if not (x1_coordinate < event.x_root < x2_coordinate and
                y1_coordinate < event.y_root < y2_coordinate):
            self.attributes("-alpha", 0.1)
            self.bell()
            self.after(100, lambda: self.attributes("-alpha", 1))

    @abstractmethod
    def body(self, _):
        """ Defines the widgets underlying in frame(_), and if possible returns the widget on
            which the focus had to be set."""
        return False

    def button_box(self):
        """ Create a new frame in top level window and add two buttons "OK" and
            "Cancel" which will call the 'ok' and 'cancel' method of this class resp.
             In many inherited class this function is override."""

        below_hz_frame = tkinter.Frame(self)
        ok_button = ttk.Button(below_hz_frame, text="OK",
                               width=10, command=self.ok,
                               default=tkinter.ACTIVE)
        ok_button.grid(row=0, column=0, padx=30, pady=10)
        cancel_button = ttk.Button(below_hz_frame, text="Cancel", width=10,
                                   command=self.cancel)
        cancel_button.grid(row=0, column=1, padx=30, pady=10)

        # bind 'ok' method to the 'enter' button of the keyboard
        self.bind("<Return>", self.ok)

        # bind 'cancel' method to the 'esc' button of the keyboard
        self.bind("<Escape>", self.cancel)
        below_hz_frame.pack(fill=tkinter.X)

    def ok(self, _=None):
        """ This will destroy the top level window and allow to do some task after user
            done their work before closing the window."""

        if not self.validate():
            self.initial_focus.focus_set()  # put focus back
            return
        self.withdraw()
        self.update_idletasks()
        self.temp.set(True)  # set boolean variable temp equal to True
        self.apply()
        self.parent.focus_set()
        self.destroy()

    def cancel(self, _=None):
        """ This will destroy the top level window"""

        self.parent.focus_set()
        self.temp.set(False)  # set boolean variable temp equal to False
        self.destroy()

    @abstractmethod
    def validate(self):
        """ Validate the data, it is inherited in add_profile class."""
        return 1

    @abstractmethod
    def apply(self):
        """ Called after user successfully terminate the top-level window"""
        return


class QuitWindow(BasicDialog):
    """ This class will ask user if he/she really want to quit."""

    def __init__(self, parent):
        BasicDialog.__init__(self, parent, title=None)

    def body(self, master):
        path_dir = os.path.dirname(os.path.realpath(__file__))
        image_ssid_entry = tkinter.PhotoImage(file=path_dir + "/data/images/warning.png")
        image_label = tkinter.Label(master, image=image_ssid_entry, bg="white")
        image_label.image = image_ssid_entry
        text_label = tkinter.Label(master, text="Do you really wish to Quit?",
                                   bg="white", font=("Playfair Display", 12))
        image_label.grid(row=0, column=0, sticky="w", padx=5, pady=(8, 2))
        text_label.grid(row=0, column=1, sticky="w", padx=(10, 5), pady=(8, 2))

    def validate(self):
        """ Validate the data, it is inherited in add_profile class."""
        return 1

    def apply(self):
        """ Called after user successfully terminate the top-level window"""
        return


class AddProfile(BasicDialog):
    """ In this the user had to enter all the valid details like
    SSID's, authentication, etc. to add wi-fi profile to the system."""

    def __init__(self, parent, app_path):
        """ All the string variable of the entry widget and combobox(drop-down list)
            are defined """
        self.app_path = app_path
        self.ssid = tkinter.StringVar()
        self.connection_mode = tkinter.StringVar()
        self.authentication = tkinter.StringVar()
        self.encryption = tkinter.StringVar()
        self.password = tkinter.StringVar()
        self.profile_data = []
        # calling constructor of base class.
        BasicDialog.__init__(self, parent, title=None)

    def body(self, master):

        self.ssid.set("")
        self.connection_mode.set("Auto")
        self.authentication.set("WPA2PSK")
        self.encryption.set("AES")
        self.password.set("")
        temp_text = "Not available for ad-hoc and WEP (only WPA and WPA2 personnel)"
        top_label = tkinter.Label(master, text=temp_text, background="white",
                                  font=("Playfair Display", 8, "bold"))
        self.ssid_label = tkinter.Label(master, text="SSID", background="white",
                                        font=("Playfair Display", 10))
        ssid_entry = ttk.Entry(master, cursor="xterm", textvariable=self.ssid, width=28)
        cm_label = tkinter.Label(master, text="Connection Mode", background="white",
                                 font=("Playfair Display", 10))
        cm_combo = ttk.Combobox(master, textvariable=self.connection_mode,
                                values=["Auto", "Manual"], state="readonly")
        auth_label = tkinter.Label(master, text="Authentication", background="white",
                                   font=("Playfair Display", 10))
        auth_combo = ttk.Combobox(master, textvariable=self.authentication,
                                  values=["WPA2PSK", "WPAPSK", "open"], state="readonly")
        encrypt_label = tkinter.Label(master, text="Encryption", background="white",
                                      font=("Playfair Display", 10))
        encrypt_combo = ttk.Combobox(master, textvariable=self.encryption,
                                     values=["AES", "TKIP"], state="readonly")
        self.pass_label = tkinter.Label(master, text="Password", background="white",
                                        font=("Playfair Display", 10))
        self.pass_entry = ttk.Entry(master, cursor="xterm", textvariable=self.password,
                                    show="*", width=28)
        top_label.grid(row=0, column=0, columnspan=2, padx=2, pady=(8, 2))
        self.ssid_label.grid(row=1, column=0, sticky="w", padx=5, pady=(8, 2))
        ssid_entry.grid(row=1, column=1, sticky="w", padx=(10, 5), pady=(8, 2))
        cm_label.grid(row=2, column=0, sticky="w", padx=5, pady=(8, 2))
        cm_combo.grid(row=2, column=1, sticky="w", padx=(10, 5), pady=(8, 2))
        auth_label.grid(row=3, column=0, sticky="w", padx=5, pady=(8, 2))
        auth_combo.grid(row=3, column=1, sticky="w", padx=(10, 5), pady=(8, 2))
        encrypt_label.grid(row=4, column=0, sticky="w", padx=5, pady=8)
        encrypt_combo.grid(row=4, column=1, sticky="w", padx=(10, 5), pady=(8, 2))
        self.pass_label.grid(row=5, column=0, sticky="w", padx=5, pady=8)
        self.pass_entry.grid(row=5, column=1, sticky="w", padx=(10, 5), pady=(8, 2))
        bottom_label = tkinter.Label(master, text="(You can use <Tab> key to communicate "
                                                  "between entry fields)", background="white",
                                     font=("Playfair Display", 8, "italic"))
        bottom_label.grid(row=6, column=0, columnspan=2, padx=2, pady=(8, 2))
        return ssid_entry

    def validate(self):
        """ Check if user entered the correct details and if not, warn user and protect them
            from moving them to next step."""

        # Check if user leaves the 'ssid' blank.
        if self.ssid.get() == "":
            self.ssid_label.configure(text="SSID*", foreground="red")
            return

        # If authentication is not open, then password should not
        # be empty and between 8 to 64 char.
        if self.authentication.get() != "open":
            self.ssid_label.configure(text="SSID", foreground="black")
            self.initial_focus = self.pass_entry
            if self.password.get() == "":
                self.pass_label.configure(text="Password(" + self.authentication.get() + ")*",
                                          foreground="red")
                return

            if not 8 <= len(self.password.get()) <= 63:
                self.pass_label.configure(text="Password(8 to 63)*", foreground="red")
                return

            if self.authentication.get() == "WPA2PSK":
                self.encryption.set("AES")

        # If Authentication is 'open' then the password should be ""
        # and encryption to be None.
        elif self.authentication.get() == "open":
            self.encryption.set("none")
            self.password.set("")

        self.pass_label.configure(text="Password", foreground="black")

        # Creating a list of user entered field (SSID's, etc.)
        self.profile_data.append(self.ssid.get())
        self.profile_data.append(self.connection_mode.get())
        self.profile_data.append(self.authentication.get())
        self.profile_data.append(self.encryption.get())
        self.profile_data.append(self.password.get())

        self.withdraw()  # hide the top-level window until another top-level closes.

        # Creates another top-level window over this top-level window(add_profile)
        result = AddProfile2(self.parent, self.profile_data)
        self.profile_data.clear()  # clear list

        # If everything ok then return 1
        # else move focus to previous top level window(add_profile).
        if result.temp.get():
            return 1
        self.initial_focus = self
        self.deiconify()  # show the top-level window(add_profile)

    def apply(self):
        """ Create xml file and add profile to system"""

        file_name = str(sum([ord(i) for i in self.ssid.get()]))

        def saving_file(xml):
            """ Save user profile in xml format to temp_ dir."""

            xml_string = etree.tostring(xml)
            parsed = minidom.parseString(xml_string)
            with open(self.app_path + "\\temp_\\" + file_name + ".xml", "w") as file:
                file.write(parsed.toprettyxml(indent="        "))

        parse_xml = etree.parse(os.path.dirname(os.path.realpath(__file__)) +
                                "/data/sampleProfile.xml")

        # The below code will parse the sample xml file
        # and fill important details entered by the user.
        root_tree = parse_xml.getroot()
        root_tree[0].text = self.ssid.get()
        root_tree[1][0][0].text = self.ssid.get()
        root_tree[3].text = self.connection_mode.get().lower()
        security = root_tree[4][0]
        security[0][0].text = self.authentication.get()
        security[0][1].text = self.encryption.get()
        if self.authentication.get() != "open":
            etree.SubElement(security, "sharedKey")
            etree.SubElement(security[1], "keyType").text = "passPhrase"
            etree.SubElement(security[1], "protected").text = "false"
            etree.SubElement(security[1], "keyMaterial").text = self.password.get()

        # Save the xml file
        saving_file(root_tree)

        # Add profile to the system.
        temp_path = 'netsh wlan add profile filename="' + self.app_path + "\\temp_\\"
        output_ = subprocess.run(temp_path + file_name + '.xml"', shell=True,
                                 capture_output=True, text=True)
        os.remove(self.app_path + "\\temp_\\" + file_name + ".xml")

        # If unable to add profile.
        if output_.returncode != 0:
            message = "Sorry, Unable to add profile.\n(You entered wrong details " \
                      "or else you don't have admin rights.)"
            image_ = "error"

        else:
            message = "Profile added successfully (Please Refresh)"
            image_ = "warning"

        MessageBox(self.parent, message, image_)


class AddProfile2(BasicDialog):
    """ This will display user entered details and wait for confirmation from user
        for further proceeding(ie., adding to system)"""

    def __init__(self, parent, profile_data):
        self.profile_data = profile_data  # list of user entered details
        BasicDialog.__init__(self, parent, title=None)

    def body(self, master):
        """ Create widgets to display user entered details"""

        def building_block(text_, row_no, column_no, color):
            """ Defining labels and packing."""
            temp = tkinter.Label(master, text=text_, font=("Playfair Display", 13),
                                 background="white", foreground=color)
            temp.grid(row=row_no, column=column_no, sticky="nw", padx=5, pady=(8, 2))

        building_block("SSID ", 0, 0, "red")
        building_block(":  '" + self.profile_data[0] + "'", 0, 1, "green")
        building_block("Connection Type ", 1, 0, "red")
        building_block(":  " + "ESS", 1, 1, "blue")
        building_block("Connection Mode ", 2, 0, "red")
        building_block(":  " + self.profile_data[1], 2, 1, "green")
        building_block("Authentication ", 3, 0, "red")
        building_block(":  " + self.profile_data[2], 3, 1, "green")
        building_block("Encryption ", 4, 0, "red")
        building_block(":  " + self.profile_data[3], 4, 1, "green")
        building_block("Key Type ", 5, 0, "red")
        building_block(":  " + "PassPhrase", 5, 1, "blue")
        building_block("Password: ", 6, 0, "red")
        building_block(":  '" + self.profile_data[4] + "'", 6, 1, "green")

    def validate(self):
        """ Validate the data, it is inherited in add_profile class."""
        return 1

    def apply(self):
        """ Called after user successfully terminate the top-level window"""
        return


class MessageBox(BasicDialog):
    """ Display message to user for success or error"""

    def __init__(self, parent, message, image_to_displayed=None):
        """ Text or image to be displayed"""

        self.text_ = message
        path_dir = os.path.dirname(os.path.realpath(__file__))
        if image_to_displayed == "warning":
            image_ = tkinter.PhotoImage(file=path_dir + "/data/images/warning.png")
        elif image_to_displayed == 'check':
            image_ = tkinter.PhotoImage(file=path_dir + "/data/images/check.png")
        else:
            image_ = tkinter.PhotoImage(file=path_dir + "/data/images/error.png")

        self.image_ = image_
        BasicDialog.__init__(self, parent, title=None)

    def body(self, master):

        image_label = tkinter.Label(master, image=self.image_, bg="white")
        image_label.image = self.image_
        message_label = tkinter.Label(master, text=self.text_, bg="white",
                                      font=("Playfair Display", 12))
        msg_for_user = "Software is under development. " \
                       "Contact developer for software related issues(check 'About')."
        imp_msg = tkinter.Label(master, text=msg_for_user, bg="white",
                                font=("Playfair Display", 8, "italic"))
        image_label.grid(row=0, column=0, rowspan=2, sticky="w", padx=5, pady=(8, 2))
        message_label.grid(row=0, column=1, sticky="w", padx=(0, 5), pady=(8, 2))
        imp_msg.grid(row=1, column=1, sticky="w", padx=(0, 5), pady=(8, 2))

    def button_box(self):

        below_hz_frame = tkinter.Frame(self)
        ok_button = ttk.Button(below_hz_frame, text="OK", width=10,
                               command=self.ok, default=tkinter.ACTIVE)
        self.bind("<Return>", self.ok)
        ok_button.pack(side=tkinter.RIGHT, padx=15, pady=5)
        below_hz_frame.pack(fill=tkinter.X)

    def validate(self):
        """ Validate the data, it is inherited in add_profile class."""
        return 1

    def apply(self):
        """ Called after user successfully terminate the top-level window"""
        return


class About(BasicDialog):
    """ This class designs the About top-level window, it opens when
    the user user click on 'About' in menu bar in Application main window.
    """

    def __init__(self, parent):
        """ Designing the basic top level window"""

        BasicDialog.__init__(self, parent, title=None)

    def body(self, master):

        def link_click(_):
            """ Detects the tag and open the link."""

            tag_name = about_content.tag_names(tkinter.CURRENT)[0]
            about_content.tag_config(tag_name, foreground="#551A8B")
            if tag_name == 'hyper':
                webbrowser.open("https://www.facebook.com/nihal.agarwal.14")
            else:
                webbrowser.open("https://github.com/NihalAgarwal/Windows-Wi-Fi-Manager")

        def mouse_in(event):
            """ Changing the cursor if use place cursor on link."""

            if str(event.type) == 'Enter':
                about_content.config(cursor="hand2")
            else:
                about_content.config(cursor="arrow")

        about = "This is my winter holidays project which I am sharing with you all. Hope, you" \
                " understood what the Application does. It allows you to look at your saved WiFi" \
                " profile and their details (SSID, Authentication and key) of your system. I know" \
                " you can do it manually by using CMD but rather than typing 'netsh.' and search" \
                " for the profile in odd-looking CMD with complex and not memorizable command" \
                " you can do it by just running ‘.exe’ file of this Application and see the" \
                " list of all saved WiFi profile in sorted order in table form. You can also" \
                " delete the saved Wi-Fi profile from your system and most important you can" \
                " also add the 'Wi-Fi name' and 'passkey' of anyone's Wi-Fi and when your system" \
                " is in a range of that Wi-Fi, it gets connected to that network.\n" \
                "For more info go to my project GitHub page (link below)."

        # text widget 1 for about the Application does
        about_content = tkinter.Text(master, background="#107896",
                                     foreground="white", cursor='arrow', wrap=tkinter.WORD, padx=10)

        # specifying styles for different text
        about_content.tag_configure('bold_italics', foreground="black",
                                    font=('Arial', 12, 'bold', 'italic', 'underline'))
        about_content.tag_configure('normal', font=('Arial', 12), spacing2=0.5)
        about_content.tag_configure('normal1', font=('Arial', 12), foreground="red")
        about_content.tag_configure('hyper', font=('Arial', 12),
                                    underline=1, foreground="#0000FF")
        about_content.tag_configure('hyper1', font=('Arial', 12),
                                    underline=1, foreground="#0000FF")

        # binding event for mouse pointer
        about_content.tag_bind("hyper", "<Button-1>", link_click)
        about_content.tag_bind("hyper", "<Enter>", mouse_in)
        about_content.tag_bind("hyper", "<Leave>", mouse_in)
        about_content.tag_bind("hyper1", "<Button-1>", link_click)
        about_content.tag_bind("hyper1", "<Enter>", mouse_in)
        about_content.tag_bind("hyper1", "<Leave>", mouse_in)

        about_content.insert(tkinter.END, "\nAbout\n\n", "bold_italics")
        about_content.insert('end', about, "normal")

        # Adding contact information
        about_content.insert(tkinter.END, "\n\nContact\n\n", "bold_italics")

        email_info = "nihal.agarwal.1426@gmail.com\n\n"
        facebook_info = "www.facebook.com/nihal.agarwal.14"
        github_info = "https://github.com/NihalAgarwal/Windows-Wi-Fi-Manager"

        about_content.insert('end', " Email : ", 'normal1')
        about_content.insert('end', email_info, 'normal')

        about_content.insert('end', " Follow on Facebook : ", 'normal1')
        about_content.insert('end', facebook_info, 'hyper')
        about_content.insert('end', "\n\n", 'normal')

        about_content.insert('end', " Send Fork Request : ", 'normal1')
        about_content.insert('end', github_info, 'hyper1')
        about_content.insert('end', "\n\n", 'normal')

        about_content.config(state=tkinter.DISABLED)
        about_content.pack(side="top", padx=2, pady=2)

    def button_box(self):
        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

    def validate(self):
        """ Validate the data, it is inherited in add_profile class."""
        return 1

    def apply(self):
        """ Called after user successfully terminate the top-level window"""
        return


# For testing purpose
if __name__ == "__main__":
    def check_function():
        """ Call the objects method or create
        instance to check the functionality of
        any top-level widget.
        """
        About(ROOT)
        # quit_or_not(root)


    ROOT = tkinter.Tk()
    ROOT.geometry('900x500')
    ttk.Button(ROOT, text="Check", command=check_function, default=tkinter.ACTIVE).pack()
    ROOT.mainloop()
